Package: ExactTree
Type: Package
Title: Exact Tree
Version: 0.1.1
Authors@R: c(person(given = "Juan", family = "Claramunt Gonzalez", role =
        c("aut", "cre","cph"), email = "j.claramunt.gonzalez@fsw.leidenuniv.nl"),
        person("Bart Jan","van Os",role=c("aut"),email="trees@4os.eu"),
        person(given = "Elise", family = "Dusseldorp", role = c("aut"), email
        = "elise.dusseldorp@fsw.leidenuniv.nl"))
Maintainer: Juan Claramunt Gonzalez <j.claramunt.gonzalez@fsw.leidenuniv.nl>
Description: Grows optimally global trees based on the algorithm defined in "An exact dynamic programming algorithm for regression and classification trees" (2011). It is possible to obtain both classification and regression trees depending on the measurement level of the outcome variable. The algorithm is based on the dynamic programming principle and guarantees that the resulting tree is optimal with respect to the chosen impurity measure. The package also includes a function to visualize the resulting trees, a function that summarizes the tree with its splitting information and leaf information, and a predict function that provides estimates for a new dataset given a model fit. 
Depends: R (>= 3.0.2), partykit, pracma, stats, grid, utils, graphics,
        formula.tools
Imports: gridtext, DescTools, methods, rpart
License: GPL-3
Encoding: UTF-8
Author: Juan Claramunt Gonzalez [aut, cre, cph],
  Bart Jan van Os [aut],
  Elise Dusseldorp [aut]
NeedsCompilation: yes
RoxygenNote: 7.3.2
Packaged: 2026-07-29 12:23:52 UTC; jclaramunt
Repository: CRAN
Date/Publication: 2026-08-07 10:30:14 UTC
