# Generated by roxygen2: do not edit by hand

S3method(as.party,ETree)
S3method(plot,ETree)
S3method(predict,ETree)
S3method(prune,ETree)
S3method(summary,ETree)
export(ETree)
export(ETree.control)
export(SelectVar)
importFrom(DescTools,BinToDec)
importFrom(DescTools,DecToBin)
importFrom(formula.tools,lhs.vars)
importFrom(formula.tools,rhs.vars)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.layout)
importFrom(grid,grid.lines)
importFrom(grid,grid.newpage)
importFrom(grid,grid.points)
importFrom(grid,grid.polygon)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,grid.yaxis)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,unit)
importFrom(grid,upViewport)
importFrom(grid,viewport)
importFrom(gridtext,richtext_grob)
importFrom(methods,is)
importFrom(partykit,as.party)
importFrom(partykit,character_split)
importFrom(partykit,id_node)
importFrom(partykit,is.terminal)
importFrom(partykit,kids_node)
importFrom(partykit,node_party)
importFrom(partykit,nodeids)
importFrom(partykit,party)
importFrom(partykit,partynode)
importFrom(partykit,partysplit)
importFrom(partykit,plot.party)
importFrom(partykit,split_node)
importFrom(pracma,Reshape)
importFrom(pracma,fprintf)
importFrom(pracma,histc)
importFrom(pracma,isempty)
importFrom(pracma,randperm)
importFrom(pracma,repmat)
importFrom(pracma,strcmp)
importFrom(rpart,prune)
importFrom(stats,complete.cases)
importFrom(stats,model.frame)
useDynLib(ExactTree, .registration=TRUE)
