# BootsTrapYX<-function(Y,X,BootSize,WithDraw){
# SampleSize<-dim(X)[1]
# if(nargs()<4){
#   I<-BootsTrapIndex(BootSize,SampleSize)
# }else{
#   I<-BootsTrapIndex(BootSize,SampleSize,WithDraw)
# }
# X<-X[I,]
# Y<-Y[I,]
#
# return(list(Y=Y, X=X))
# }
